/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/*
 * 15/07/13 - Change notice:
 * This file has been modified by Mobius Software Ltd.
 * For more information please visit http://www.mobius.ua
 */
package ua.mobius.media.server.impl.dsp.audio.g711.alaw;

/**
 *
 * @author oifa yulian
 */
public class EncoderData {

	//encoded data , first dimension is high byte , second dimension is low byte
	public final static byte[][] aLawCompressTableChunks = new byte[256][];
	public final static byte[] aLawCompressTable = new byte[65536];
	static
	{
		init0_31();
		init32_63();
		init64_95();
		init96_127();
		init128_159();
		init160_191();
		init192_223();		
		init224_255();
		
		for(int i=0;i<256;i++)
			System.arraycopy(aLawCompressTableChunks[i], 0, aLawCompressTable, i*256, 256);
	}
	
	private static void init0_31()
	{
		aLawCompressTableChunks[0] = new byte[] {
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda
		};

		aLawCompressTableChunks[1] = new byte[] {
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca
		};

		aLawCompressTableChunks[2] = new byte[] {
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2
		};

		aLawCompressTableChunks[3] = new byte[] {
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa
		};

		aLawCompressTableChunks[4] = new byte[] {
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6
		};

		aLawCompressTableChunks[5] = new byte[] {
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2
		};

		aLawCompressTableChunks[6] = new byte[] {
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee
		};

		aLawCompressTableChunks[7] = new byte[] {
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea
		};

		aLawCompressTableChunks[8] = new byte[] {
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94
		};

		aLawCompressTableChunks[9] = new byte[] {
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96
		};

		aLawCompressTableChunks[10] = new byte[] {
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90
		};

		aLawCompressTableChunks[11] = new byte[] {
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92
		};

		aLawCompressTableChunks[12] = new byte[] {
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c
		};

		aLawCompressTableChunks[13] = new byte[] {
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e
		};

		aLawCompressTableChunks[14] = new byte[] {
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98
		};

		aLawCompressTableChunks[15] = new byte[] {
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a
		};

		aLawCompressTableChunks[16] = new byte[] {
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85
		};

		aLawCompressTableChunks[17] = new byte[] {
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84
		};

		aLawCompressTableChunks[18] = new byte[] {
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87
		};

		aLawCompressTableChunks[19] = new byte[] {
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86
		};

		aLawCompressTableChunks[20] = new byte[] {
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81
		};

		aLawCompressTableChunks[21] = new byte[] {
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80
		};

		aLawCompressTableChunks[22] = new byte[] {
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83
		};

		aLawCompressTableChunks[23] = new byte[] {
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82
		};

		aLawCompressTableChunks[24] = new byte[] {
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d
		};

		aLawCompressTableChunks[25] = new byte[] {
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c
		};

		aLawCompressTableChunks[26] = new byte[] {
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f
		};

		aLawCompressTableChunks[27] = new byte[] {
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e
		};

		aLawCompressTableChunks[28] = new byte[] {
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89
		};

		aLawCompressTableChunks[29] = new byte[] {
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88
		};

		aLawCompressTableChunks[30] = new byte[] {
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b
		};

		aLawCompressTableChunks[31] = new byte[] {
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a
		};
	}
	
	private static void init32_63()
	{
		aLawCompressTableChunks[32] = new byte[] {
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5
		};

		aLawCompressTableChunks[33] = new byte[] {
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5
		};

		aLawCompressTableChunks[34] = new byte[] {
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4
		};

		aLawCompressTableChunks[35] = new byte[] {
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4
		};

		aLawCompressTableChunks[36] = new byte[] {
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7
		};

		aLawCompressTableChunks[37] = new byte[] {
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7
		};

		aLawCompressTableChunks[38] = new byte[] {
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6
		};

		aLawCompressTableChunks[39] = new byte[] {
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6
		};

		aLawCompressTableChunks[40] = new byte[] {
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1
		};

		aLawCompressTableChunks[41] = new byte[] {
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1
		};

		aLawCompressTableChunks[42] = new byte[] {
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0
		};

		aLawCompressTableChunks[43] = new byte[] {
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0
		};

		aLawCompressTableChunks[44] = new byte[] {
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3
		};

		aLawCompressTableChunks[45] = new byte[] {
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3
		};

		aLawCompressTableChunks[46] = new byte[] {
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2
		};

		aLawCompressTableChunks[47] = new byte[] {
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2
		};

		aLawCompressTableChunks[48] = new byte[] {
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd
		};

		aLawCompressTableChunks[49] = new byte[] {
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd
		};

		aLawCompressTableChunks[50] = new byte[] {
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc
		};

		aLawCompressTableChunks[51] = new byte[] {
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc
		};

		aLawCompressTableChunks[52] = new byte[] {
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf
		};

		aLawCompressTableChunks[53] = new byte[] {
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf
		};

		aLawCompressTableChunks[54] = new byte[] {
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe
		};

		aLawCompressTableChunks[55] = new byte[] {
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe
		};

		aLawCompressTableChunks[56] = new byte[] {
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9
		};

		aLawCompressTableChunks[57] = new byte[] {
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9
		};

		aLawCompressTableChunks[58] = new byte[] {
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8
		};

		aLawCompressTableChunks[59] = new byte[] {
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8
		};

		aLawCompressTableChunks[60] = new byte[] {
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb
		};

		aLawCompressTableChunks[61] = new byte[] {
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb
		};

		aLawCompressTableChunks[62] = new byte[] {
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba
		};

		aLawCompressTableChunks[63] = new byte[] {
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba
		};
	}
	
	private static void init64_95()
	{
		aLawCompressTableChunks[64] = new byte[] {
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5
		};

		aLawCompressTableChunks[65] = new byte[] {
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5
		};

		aLawCompressTableChunks[66] = new byte[] {
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5
		};

		aLawCompressTableChunks[67] = new byte[] {
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5
		};

		aLawCompressTableChunks[68] = new byte[] {
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4
		};

		aLawCompressTableChunks[69] = new byte[] {
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4
		};

		aLawCompressTableChunks[70] = new byte[] {
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4
		};

		aLawCompressTableChunks[71] = new byte[] {
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4
		};

		aLawCompressTableChunks[72] = new byte[] {
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7
		};

		aLawCompressTableChunks[73] = new byte[] {
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7
		};

		aLawCompressTableChunks[74] = new byte[] {
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7
		};

		aLawCompressTableChunks[75] = new byte[] {
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7
		};

		aLawCompressTableChunks[76] = new byte[] {
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6
		};

		aLawCompressTableChunks[77] = new byte[] {
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6
		};

		aLawCompressTableChunks[78] = new byte[] {
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6
		};

		aLawCompressTableChunks[79] = new byte[] {
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6
		};

		aLawCompressTableChunks[80] = new byte[] {
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1
		};

		aLawCompressTableChunks[81] = new byte[] {
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1
		};

		aLawCompressTableChunks[82] = new byte[] {
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1
		};

		aLawCompressTableChunks[83] = new byte[] {
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1
		};

		aLawCompressTableChunks[84] = new byte[] {
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0
		};

		aLawCompressTableChunks[85] = new byte[] {
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0
		};

		aLawCompressTableChunks[86] = new byte[] {
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0
		};

		aLawCompressTableChunks[87] = new byte[] {
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0
		};

		aLawCompressTableChunks[88] = new byte[] {
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3
		};

		aLawCompressTableChunks[89] = new byte[] {
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3
		};

		aLawCompressTableChunks[90] = new byte[] {
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3
		};

		aLawCompressTableChunks[91] = new byte[] {
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3
		};

		aLawCompressTableChunks[92] = new byte[] {
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2
		};

		aLawCompressTableChunks[93] = new byte[] {
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2
		};

		aLawCompressTableChunks[94] = new byte[] {
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2
		};

		aLawCompressTableChunks[95] = new byte[] {
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2
		};
	}
	
	private static void init96_127()
	{
		aLawCompressTableChunks[96] = new byte[] {
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad
		};

		aLawCompressTableChunks[97] = new byte[] {
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad
		};

		aLawCompressTableChunks[98] = new byte[] {
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad
		};

		aLawCompressTableChunks[99] = new byte[] {
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad
		};

		aLawCompressTableChunks[100] = new byte[] {
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac
		};

		aLawCompressTableChunks[101] = new byte[] {
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac
		};

		aLawCompressTableChunks[102] = new byte[] {
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac
		};

		aLawCompressTableChunks[103] = new byte[] {
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac
		};

		aLawCompressTableChunks[104] = new byte[] {
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf
		};

		aLawCompressTableChunks[105] = new byte[] {
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf
		};

		aLawCompressTableChunks[106] = new byte[] {
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf
		};

		aLawCompressTableChunks[107] = new byte[] {
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf
		};

		aLawCompressTableChunks[108] = new byte[] {
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae
		};

		aLawCompressTableChunks[109] = new byte[] {
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae
		};

		aLawCompressTableChunks[110] = new byte[] {
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae
		};

		aLawCompressTableChunks[111] = new byte[] {
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae
		};

		aLawCompressTableChunks[112] = new byte[] {
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9
		};

		aLawCompressTableChunks[113] = new byte[] {
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9
		};

		aLawCompressTableChunks[114] = new byte[] {
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9
		};

		aLawCompressTableChunks[115] = new byte[] {
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9
		};

		aLawCompressTableChunks[116] = new byte[] {
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8
		};

		aLawCompressTableChunks[117] = new byte[] {
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8
		};

		aLawCompressTableChunks[118] = new byte[] {
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8
		};

		aLawCompressTableChunks[119] = new byte[] {
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8
		};

		aLawCompressTableChunks[120] = new byte[] {
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab
		};

		aLawCompressTableChunks[121] = new byte[] {
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab
		};

		aLawCompressTableChunks[122] = new byte[] {
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab
		};

		aLawCompressTableChunks[123] = new byte[] {
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab
		};

		aLawCompressTableChunks[124] = new byte[] {
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa
		};

		aLawCompressTableChunks[125] = new byte[] {
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa
		};

		aLawCompressTableChunks[126] = new byte[] {
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa
		};

		aLawCompressTableChunks[127] = new byte[] {
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa
		};
	}
	
	private static void init128_159()
	{
		aLawCompressTableChunks[128] = new byte[] {
			(byte)0x55,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a
		};

		aLawCompressTableChunks[129] = new byte[] {
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a
		};

		aLawCompressTableChunks[130] = new byte[] {
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a
		};

		aLawCompressTableChunks[131] = new byte[] {
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a
		};

		aLawCompressTableChunks[132] = new byte[] {
			(byte)0x2a,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b
		};

		aLawCompressTableChunks[133] = new byte[] {
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b
		};

		aLawCompressTableChunks[134] = new byte[] {
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b
		};

		aLawCompressTableChunks[135] = new byte[] {
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b
		};

		aLawCompressTableChunks[136] = new byte[] {
			(byte)0x2b,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28
		};

		aLawCompressTableChunks[137] = new byte[] {
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28
		};

		aLawCompressTableChunks[138] = new byte[] {
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28
		};

		aLawCompressTableChunks[139] = new byte[] {
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28
		};

		aLawCompressTableChunks[140] = new byte[] {
			(byte)0x28,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29
		};

		aLawCompressTableChunks[141] = new byte[] {
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29
		};

		aLawCompressTableChunks[142] = new byte[] {
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29
		};

		aLawCompressTableChunks[143] = new byte[] {
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29
		};

		aLawCompressTableChunks[144] = new byte[] {
			(byte)0x29,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e
		};

		aLawCompressTableChunks[145] = new byte[] {
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e
		};

		aLawCompressTableChunks[146] = new byte[] {
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e
		};

		aLawCompressTableChunks[147] = new byte[] {
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e
		};

		aLawCompressTableChunks[148] = new byte[] {
			(byte)0x2e,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f
		};

		aLawCompressTableChunks[149] = new byte[] {
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f
		};

		aLawCompressTableChunks[150] = new byte[] {
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f
		};

		aLawCompressTableChunks[151] = new byte[] {
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f
		};

		aLawCompressTableChunks[152] = new byte[] {
			(byte)0x2f,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c
		};

		aLawCompressTableChunks[153] = new byte[] {
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c
		};

		aLawCompressTableChunks[154] = new byte[] {
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c
		};

		aLawCompressTableChunks[155] = new byte[] {
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c
		};

		aLawCompressTableChunks[156] = new byte[] {
			(byte)0x2c,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d
		};

		aLawCompressTableChunks[157] = new byte[] {
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d
		};

		aLawCompressTableChunks[158] = new byte[] {
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d
		};

		aLawCompressTableChunks[159] = new byte[] {
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d
		};
	}
	
	private static void init160_191()
	{
		aLawCompressTableChunks[160] = new byte[] {
			(byte)0x2d,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22
		};

		aLawCompressTableChunks[161] = new byte[] {
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22
		};

		aLawCompressTableChunks[162] = new byte[] {
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22
		};

		aLawCompressTableChunks[163] = new byte[] {
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22
		};

		aLawCompressTableChunks[164] = new byte[] {
			(byte)0x22,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23
		};

		aLawCompressTableChunks[165] = new byte[] {
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23
		};

		aLawCompressTableChunks[166] = new byte[] {
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23
		};

		aLawCompressTableChunks[167] = new byte[] {
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23
		};

		aLawCompressTableChunks[168] = new byte[] {
			(byte)0x23,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20
		};

		aLawCompressTableChunks[169] = new byte[] {
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20
		};

		aLawCompressTableChunks[170] = new byte[] {
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20
		};

		aLawCompressTableChunks[171] = new byte[] {
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20
		};

		aLawCompressTableChunks[172] = new byte[] {
			(byte)0x20,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21
		};

		aLawCompressTableChunks[173] = new byte[] {
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21
		};

		aLawCompressTableChunks[174] = new byte[] {
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21
		};

		aLawCompressTableChunks[175] = new byte[] {
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21
		};

		aLawCompressTableChunks[176] = new byte[] {
			(byte)0x21,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26
		};

		aLawCompressTableChunks[177] = new byte[] {
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26
		};

		aLawCompressTableChunks[178] = new byte[] {
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26
		};

		aLawCompressTableChunks[179] = new byte[] {
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26
		};

		aLawCompressTableChunks[180] = new byte[] {
			(byte)0x26,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27
		};

		aLawCompressTableChunks[181] = new byte[] {
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27
		};

		aLawCompressTableChunks[182] = new byte[] {
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27
		};

		aLawCompressTableChunks[183] = new byte[] {
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27
		};

		aLawCompressTableChunks[184] = new byte[] {
			(byte)0x27,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24
		};

		aLawCompressTableChunks[185] = new byte[] {
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24
		};

		aLawCompressTableChunks[186] = new byte[] {
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24
		};

		aLawCompressTableChunks[187] = new byte[] {
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24
		};

		aLawCompressTableChunks[188] = new byte[] {
			(byte)0x24,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25
		};

		aLawCompressTableChunks[189] = new byte[] {
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25
		};

		aLawCompressTableChunks[190] = new byte[] {
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25
		};

		aLawCompressTableChunks[191] = new byte[] {
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25
		};
	}
	
	private static void init192_223()
	{
		aLawCompressTableChunks[192] = new byte[] {
			(byte)0x25,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a
		};

		aLawCompressTableChunks[193] = new byte[] {
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a
		};

		aLawCompressTableChunks[194] = new byte[] {
			(byte)0x3a,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b
		};

		aLawCompressTableChunks[195] = new byte[] {
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b
		};

		aLawCompressTableChunks[196] = new byte[] {
			(byte)0x3b,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38
		};

		aLawCompressTableChunks[197] = new byte[] {
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38
		};

		aLawCompressTableChunks[198] = new byte[] {
			(byte)0x38,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39
		};

		aLawCompressTableChunks[199] = new byte[] {
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39
		};

		aLawCompressTableChunks[200] = new byte[] {
			(byte)0x39,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e
		};

		aLawCompressTableChunks[201] = new byte[] {
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e
		};

		aLawCompressTableChunks[202] = new byte[] {
			(byte)0x3e,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f
		};

		aLawCompressTableChunks[203] = new byte[] {
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f
		};

		aLawCompressTableChunks[204] = new byte[] {
			(byte)0x3f,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c
		};

		aLawCompressTableChunks[205] = new byte[] {
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c
		};

		aLawCompressTableChunks[206] = new byte[] {
			(byte)0x3c,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d
		};

		aLawCompressTableChunks[207] = new byte[] {
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d
		};

		aLawCompressTableChunks[208] = new byte[] {
			(byte)0x3d,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32
		};

		aLawCompressTableChunks[209] = new byte[] {
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32
		};

		aLawCompressTableChunks[210] = new byte[] {
			(byte)0x32,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33
		};

		aLawCompressTableChunks[211] = new byte[] {
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33
		};

		aLawCompressTableChunks[212] = new byte[] {
			(byte)0x33,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30
		};

		aLawCompressTableChunks[213] = new byte[] {
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30
		};

		aLawCompressTableChunks[214] = new byte[] {
			(byte)0x30,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31
		};

		aLawCompressTableChunks[215] = new byte[] {
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31
		};

		aLawCompressTableChunks[216] = new byte[] {
			(byte)0x31,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36
		};

		aLawCompressTableChunks[217] = new byte[] {
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36
		};

		aLawCompressTableChunks[218] = new byte[] {
			(byte)0x36,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37
		};

		aLawCompressTableChunks[219] = new byte[] {
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37
		};

		aLawCompressTableChunks[220] = new byte[] {
			(byte)0x37,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34
		};

		aLawCompressTableChunks[221] = new byte[] {
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34
		};

		aLawCompressTableChunks[222] = new byte[] {
			(byte)0x34,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35
		};

		aLawCompressTableChunks[223] = new byte[] {
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35
		};
	}
	
	private static void init224_255()
	{
		aLawCompressTableChunks[224] = new byte[] {
			(byte)0x35,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa
		};

		aLawCompressTableChunks[225] = new byte[] {
			(byte)0xa,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb
		};

		aLawCompressTableChunks[226] = new byte[] {
			(byte)0xb,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8
		};

		aLawCompressTableChunks[227] = new byte[] {
			(byte)0x8,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9
		};

		aLawCompressTableChunks[228] = new byte[] {
			(byte)0x9,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe
		};

		aLawCompressTableChunks[229] = new byte[] {
			(byte)0xe,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf
		};

		aLawCompressTableChunks[230] = new byte[] {
			(byte)0xf,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc
		};

		aLawCompressTableChunks[231] = new byte[] {
			(byte)0xc,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd
		};

		aLawCompressTableChunks[232] = new byte[] {
			(byte)0xd,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2
		};

		aLawCompressTableChunks[233] = new byte[] {
			(byte)0x2,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3
		};

		aLawCompressTableChunks[234] = new byte[] {
			(byte)0x3,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0
		};

		aLawCompressTableChunks[235] = new byte[] {
			(byte)0x0,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1
		};

		aLawCompressTableChunks[236] = new byte[] {
			(byte)0x1,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6
		};

		aLawCompressTableChunks[237] = new byte[] {
			(byte)0x6,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7
		};

		aLawCompressTableChunks[238] = new byte[] {
			(byte)0x7,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4
		};

		aLawCompressTableChunks[239] = new byte[] {
			(byte)0x4,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5
		};

		aLawCompressTableChunks[240] = new byte[] {
			(byte)0x5,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b
		};

		aLawCompressTableChunks[241] = new byte[] {
			(byte)0x1b,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19
		};

		aLawCompressTableChunks[242] = new byte[] {
			(byte)0x19,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f
		};

		aLawCompressTableChunks[243] = new byte[] {
			(byte)0x1f,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d
		};

		aLawCompressTableChunks[244] = new byte[] {
			(byte)0x1d,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13
		};

		aLawCompressTableChunks[245] = new byte[] {
			(byte)0x13,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11
		};

		aLawCompressTableChunks[246] = new byte[] {
			(byte)0x11,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17
		};

		aLawCompressTableChunks[247] = new byte[] {
			(byte)0x17,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15
		};

		aLawCompressTableChunks[248] = new byte[] {
			(byte)0x15,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69
		};

		aLawCompressTableChunks[249] = new byte[] {
			(byte)0x69,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d
		};

		aLawCompressTableChunks[250] = new byte[] {
			(byte)0x6d,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61
		};

		aLawCompressTableChunks[251] = new byte[] {
			(byte)0x61,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65
		};

		aLawCompressTableChunks[252] = new byte[] {
			(byte)0x65,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,
			(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,
			(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,
			(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,
			(byte)0x7f,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,
			(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,
			(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,
			(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d
		};

		aLawCompressTableChunks[253] = new byte[] {
			(byte)0x7d,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,
			(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,
			(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,
			(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75
		};

		aLawCompressTableChunks[254] = new byte[] {
			(byte)0x75,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45
		};

		aLawCompressTableChunks[255] = new byte[] {
			(byte)0x45,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,
			(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,
			(byte)0x5b,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,
			(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,
			(byte)0x53,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,
			(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55
		};
	}
}

