/*
 * JBoss, Home of Professional Open Source
 * Copyright 2011, Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

/*
 * 15/07/13 - Change notice:
 * This file has been modified by Mobius Software Ltd.
 * For more information please visit http://www.mobius.ua
 */
package ua.mobius.media.server.impl.dsp.audio.g711.ulaw;

/**
 *
 * @author oifa yulian
 */
public class EncoderData {

	//encoded data , first dimension is high byte , second dimension is low byte
	public final static byte[][] muLawCompressTableChunks = new byte[256][];
	public final static byte[] muLawCompressTable = new byte[65536];
	
	static
	{
		init0_31();
		init32_63();
		init64_95();
		init96_127();
		init128_159();
		init160_191();
		init192_223();
		init224_255();
		
		for(int i=0;i<256;i++)
			System.arraycopy(muLawCompressTableChunks[i], 0, muLawCompressTable, i*256, 256);
	}
	
	private static void init0_31()
	{
		muLawCompressTableChunks[0] = new byte[] {
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,
			(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xef,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,
			(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xee,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,
			(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xed,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,
			(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xec,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,
			(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xeb,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,
			(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xea,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,
			(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe9,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,
			(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe8,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7
		};

		muLawCompressTableChunks[1] = new byte[] {
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,
			(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe7,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,
			(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe6,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,
			(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe5,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,
			(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe4,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,
			(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe3,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,
			(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe2,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,
			(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe1,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,
			(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xe0,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,
			(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xdf,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,
			(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xde,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,
			(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdd,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,
			(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdc,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb
		};

		muLawCompressTableChunks[2] = new byte[] {
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,
			(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xdb,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,
			(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xda,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,
			(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd9,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,
			(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd8,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,
			(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd7,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,
			(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd6,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,
			(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd5,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,
			(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd4,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3
		};

		muLawCompressTableChunks[3] = new byte[] {
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,
			(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd3,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,
			(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd2,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,
			(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd1,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,
			(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xd0,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,
			(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xcf,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,
			(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xce,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd
		};

		muLawCompressTableChunks[4] = new byte[] {
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,
			(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcd,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,
			(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcc,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,
			(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xcb,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,
			(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xca,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9
		};

		muLawCompressTableChunks[5] = new byte[] {
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,
			(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc9,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,
			(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc8,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,
			(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc7,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,
			(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc6,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5
		};

		muLawCompressTableChunks[6] = new byte[] {
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,
			(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc5,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,
			(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc4,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,
			(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc3,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,
			(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc2,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1
		};

		muLawCompressTableChunks[7] = new byte[] {
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,
			(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc1,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,
			(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xc0,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,
			(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbf,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe
		};

		muLawCompressTableChunks[8] = new byte[] {
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,
			(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbe,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,
			(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbd,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc
		};

		muLawCompressTableChunks[9] = new byte[] {
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,
			(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbc,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,
			(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xbb,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba
		};

		muLawCompressTableChunks[10] = new byte[] {
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,
			(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xba,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,
			(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb9,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8
		};

		muLawCompressTableChunks[11] = new byte[] {
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,
			(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb8,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,
			(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb7,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6
		};

		muLawCompressTableChunks[12] = new byte[] {
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,
			(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb6,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,
			(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb5,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4
		};

		muLawCompressTableChunks[13] = new byte[] {
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,
			(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb4,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,
			(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb3,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2
		};

		muLawCompressTableChunks[14] = new byte[] {
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,
			(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb2,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,
			(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb1,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0
		};

		muLawCompressTableChunks[15] = new byte[] {
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,
			(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xb0,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf
		};

		muLawCompressTableChunks[16] = new byte[] {
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,
			(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xaf,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae
		};

		muLawCompressTableChunks[17] = new byte[] {
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,
			(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xae,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad
		};

		muLawCompressTableChunks[18] = new byte[] {
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,
			(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xad,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac
		};

		muLawCompressTableChunks[19] = new byte[] {
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,
			(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xac,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab
		};

		muLawCompressTableChunks[20] = new byte[] {
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,
			(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xab,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa
		};

		muLawCompressTableChunks[21] = new byte[] {
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,
			(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xaa,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9
		};

		muLawCompressTableChunks[22] = new byte[] {
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,
			(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa9,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8
		};

		muLawCompressTableChunks[23] = new byte[] {
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,
			(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa8,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7
		};

		muLawCompressTableChunks[24] = new byte[] {
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,
			(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa7,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6
		};

		muLawCompressTableChunks[25] = new byte[] {
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,
			(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa6,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5
		};

		muLawCompressTableChunks[26] = new byte[] {
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,
			(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa5,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4
		};

		muLawCompressTableChunks[27] = new byte[] {
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,
			(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa4,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3
		};

		muLawCompressTableChunks[28] = new byte[] {
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,
			(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa3,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2
		};

		muLawCompressTableChunks[29] = new byte[] {
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,
			(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa2,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1
		};

		muLawCompressTableChunks[30] = new byte[] {
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,
			(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa1,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0
		};

		muLawCompressTableChunks[31] = new byte[] {
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,
			(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0xa0,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f
		};
	}
	
	private static void init32_63()
	{
		muLawCompressTableChunks[32] = new byte[] {
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f
		};

		muLawCompressTableChunks[33] = new byte[] {
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,
			(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9f,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e
		};

		muLawCompressTableChunks[34] = new byte[] {
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e
		};

		muLawCompressTableChunks[35] = new byte[] {
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,
			(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9e,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d
		};

		muLawCompressTableChunks[36] = new byte[] {
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d
		};

		muLawCompressTableChunks[37] = new byte[] {
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,
			(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9d,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c
		};

		muLawCompressTableChunks[38] = new byte[] {
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c
		};

		muLawCompressTableChunks[39] = new byte[] {
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,
			(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9c,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b
		};

		muLawCompressTableChunks[40] = new byte[] {
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b
		};

		muLawCompressTableChunks[41] = new byte[] {
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,
			(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9b,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a
		};

		muLawCompressTableChunks[42] = new byte[] {
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a
		};

		muLawCompressTableChunks[43] = new byte[] {
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,
			(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x9a,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99
		};

		muLawCompressTableChunks[44] = new byte[] {
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99
		};

		muLawCompressTableChunks[45] = new byte[] {
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,
			(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x99,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98
		};

		muLawCompressTableChunks[46] = new byte[] {
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98
		};

		muLawCompressTableChunks[47] = new byte[] {
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,
			(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x98,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97
		};

		muLawCompressTableChunks[48] = new byte[] {
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97
		};

		muLawCompressTableChunks[49] = new byte[] {
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,
			(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x97,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96
		};

		muLawCompressTableChunks[50] = new byte[] {
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96
		};

		muLawCompressTableChunks[51] = new byte[] {
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,
			(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x96,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95
		};

		muLawCompressTableChunks[52] = new byte[] {
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95
		};

		muLawCompressTableChunks[53] = new byte[] {
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,
			(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x95,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94
		};

		muLawCompressTableChunks[54] = new byte[] {
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94
		};

		muLawCompressTableChunks[55] = new byte[] {
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,
			(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x94,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93
		};

		muLawCompressTableChunks[56] = new byte[] {
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93
		};

		muLawCompressTableChunks[57] = new byte[] {
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,
			(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x93,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92
		};

		muLawCompressTableChunks[58] = new byte[] {
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92
		};

		muLawCompressTableChunks[59] = new byte[] {
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,
			(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x92,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91
		};

		muLawCompressTableChunks[60] = new byte[] {
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91
		};

		muLawCompressTableChunks[61] = new byte[] {
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,
			(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x91,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90
		};

		muLawCompressTableChunks[62] = new byte[] {
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90
		};

		muLawCompressTableChunks[63] = new byte[] {
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,
			(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x90,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f
		};
	}
	
	private static void init64_95()
	{
		muLawCompressTableChunks[64] = new byte[] {
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f
		};

		muLawCompressTableChunks[65] = new byte[] {
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f
		};

		muLawCompressTableChunks[66] = new byte[] {
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f
		};

		muLawCompressTableChunks[67] = new byte[] {
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,
			(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8f,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e
		};

		muLawCompressTableChunks[68] = new byte[] {
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e
		};

		muLawCompressTableChunks[69] = new byte[] {
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e
		};

		muLawCompressTableChunks[70] = new byte[] {
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e
		};

		muLawCompressTableChunks[71] = new byte[] {
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,
			(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8e,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d
		};

		muLawCompressTableChunks[72] = new byte[] {
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d
		};

		muLawCompressTableChunks[73] = new byte[] {
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d
		};

		muLawCompressTableChunks[74] = new byte[] {
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d
		};

		muLawCompressTableChunks[75] = new byte[] {
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,
			(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8d,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c
		};

		muLawCompressTableChunks[76] = new byte[] {
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c
		};

		muLawCompressTableChunks[77] = new byte[] {
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c
		};

		muLawCompressTableChunks[78] = new byte[] {
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c
		};

		muLawCompressTableChunks[79] = new byte[] {
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,
			(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8c,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b
		};

		muLawCompressTableChunks[80] = new byte[] {
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b
		};

		muLawCompressTableChunks[81] = new byte[] {
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b
		};

		muLawCompressTableChunks[82] = new byte[] {
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b
		};

		muLawCompressTableChunks[83] = new byte[] {
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,
			(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8b,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a
		};

		muLawCompressTableChunks[84] = new byte[] {
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a
		};

		muLawCompressTableChunks[85] = new byte[] {
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a
		};

		muLawCompressTableChunks[86] = new byte[] {
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a
		};

		muLawCompressTableChunks[87] = new byte[] {
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,
			(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x8a,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89
		};

		muLawCompressTableChunks[88] = new byte[] {
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89
		};

		muLawCompressTableChunks[89] = new byte[] {
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89
		};

		muLawCompressTableChunks[90] = new byte[] {
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89
		};

		muLawCompressTableChunks[91] = new byte[] {
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,
			(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x89,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88
		};

		muLawCompressTableChunks[92] = new byte[] {
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88
		};

		muLawCompressTableChunks[93] = new byte[] {
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88
		};

		muLawCompressTableChunks[94] = new byte[] {
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88
		};

		muLawCompressTableChunks[95] = new byte[] {
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,
			(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x88,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87
		};
	}
	
	private static void init96_127()
	{
		muLawCompressTableChunks[96] = new byte[] {
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87
		};

		muLawCompressTableChunks[97] = new byte[] {
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87
		};

		muLawCompressTableChunks[98] = new byte[] {
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87
		};

		muLawCompressTableChunks[99] = new byte[] {
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,
			(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x87,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86
		};

		muLawCompressTableChunks[100] = new byte[] {
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86
		};

		muLawCompressTableChunks[101] = new byte[] {
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86
		};

		muLawCompressTableChunks[102] = new byte[] {
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86
		};

		muLawCompressTableChunks[103] = new byte[] {
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,
			(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x86,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85
		};

		muLawCompressTableChunks[104] = new byte[] {
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85
		};

		muLawCompressTableChunks[105] = new byte[] {
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85
		};

		muLawCompressTableChunks[106] = new byte[] {
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85
		};

		muLawCompressTableChunks[107] = new byte[] {
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,
			(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x85,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84
		};

		muLawCompressTableChunks[108] = new byte[] {
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84
		};

		muLawCompressTableChunks[109] = new byte[] {
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84
		};

		muLawCompressTableChunks[110] = new byte[] {
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84
		};

		muLawCompressTableChunks[111] = new byte[] {
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,
			(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x84,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83
		};

		muLawCompressTableChunks[112] = new byte[] {
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83
		};

		muLawCompressTableChunks[113] = new byte[] {
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83
		};

		muLawCompressTableChunks[114] = new byte[] {
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83
		};

		muLawCompressTableChunks[115] = new byte[] {
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,
			(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x83,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82
		};

		muLawCompressTableChunks[116] = new byte[] {
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82
		};

		muLawCompressTableChunks[117] = new byte[] {
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82
		};

		muLawCompressTableChunks[118] = new byte[] {
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82
		};

		muLawCompressTableChunks[119] = new byte[] {
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,
			(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x82,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81
		};

		muLawCompressTableChunks[120] = new byte[] {
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81
		};

		muLawCompressTableChunks[121] = new byte[] {
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81
		};

		muLawCompressTableChunks[122] = new byte[] {
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81
		};

		muLawCompressTableChunks[123] = new byte[] {
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,
			(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x81,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80
		};

		muLawCompressTableChunks[124] = new byte[] {
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80
		};

		muLawCompressTableChunks[125] = new byte[] {
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80
		};

		muLawCompressTableChunks[126] = new byte[] {
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80
		};

		muLawCompressTableChunks[127] = new byte[] {
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,
			(byte)0x80,(byte)0x80,(byte)0x80,(byte)0x80,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,
			(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,
			(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfe,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,
			(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfd,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,
			(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfc,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,
			(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfb,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,
			(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xfa,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,
			(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf9,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,
			(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf8,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,
			(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf7,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,
			(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf6,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,
			(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf5,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,
			(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf4,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,
			(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf3,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,
			(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf2,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,
			(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf1,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,
			(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xf0,(byte)0xff,(byte)0xff,(byte)0xff,(byte)0xff
		};
	}
	
	private static void init128_159()
	{
		muLawCompressTableChunks[128] = new byte[] {
			(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x75,(byte)0x75,(byte)0x75,
			(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7d,(byte)0x7d,(byte)0x7d,
			(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7f,(byte)0x7f,(byte)0x7f,
			(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x7f,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0
		};

		muLawCompressTableChunks[129] = new byte[] {
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0
		};

		muLawCompressTableChunks[130] = new byte[] {
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0
		};

		muLawCompressTableChunks[131] = new byte[] {
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0
		};

		muLawCompressTableChunks[132] = new byte[] {
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,
			(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x0,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1
		};

		muLawCompressTableChunks[133] = new byte[] {
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1
		};

		muLawCompressTableChunks[134] = new byte[] {
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1
		};

		muLawCompressTableChunks[135] = new byte[] {
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1
		};

		muLawCompressTableChunks[136] = new byte[] {
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,
			(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x1,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2
		};

		muLawCompressTableChunks[137] = new byte[] {
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2
		};

		muLawCompressTableChunks[138] = new byte[] {
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2
		};

		muLawCompressTableChunks[139] = new byte[] {
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2
		};

		muLawCompressTableChunks[140] = new byte[] {
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,
			(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x2,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3
		};

		muLawCompressTableChunks[141] = new byte[] {
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3
		};

		muLawCompressTableChunks[142] = new byte[] {
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3
		};

		muLawCompressTableChunks[143] = new byte[] {
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3
		};

		muLawCompressTableChunks[144] = new byte[] {
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,
			(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x3,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4
		};

		muLawCompressTableChunks[145] = new byte[] {
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4
		};

		muLawCompressTableChunks[146] = new byte[] {
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4
		};

		muLawCompressTableChunks[147] = new byte[] {
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4
		};

		muLawCompressTableChunks[148] = new byte[] {
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,
			(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x4,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5
		};

		muLawCompressTableChunks[149] = new byte[] {
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5
		};

		muLawCompressTableChunks[150] = new byte[] {
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5
		};

		muLawCompressTableChunks[151] = new byte[] {
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5
		};

		muLawCompressTableChunks[152] = new byte[] {
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,
			(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x5,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6
		};

		muLawCompressTableChunks[153] = new byte[] {
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6
		};

		muLawCompressTableChunks[154] = new byte[] {
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6
		};

		muLawCompressTableChunks[155] = new byte[] {
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6
		};

		muLawCompressTableChunks[156] = new byte[] {
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,
			(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x6,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7
		};

		muLawCompressTableChunks[157] = new byte[] {
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7
		};

		muLawCompressTableChunks[158] = new byte[] {
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7
		};

		muLawCompressTableChunks[159] = new byte[] {
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7
		};
	}
	
	private static void init160_191()
	{
		muLawCompressTableChunks[160] = new byte[] {
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,
			(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x7,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8
		};

		muLawCompressTableChunks[161] = new byte[] {
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8
		};

		muLawCompressTableChunks[162] = new byte[] {
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8
		};

		muLawCompressTableChunks[163] = new byte[] {
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8
		};

		muLawCompressTableChunks[164] = new byte[] {
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,
			(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x8,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9
		};

		muLawCompressTableChunks[165] = new byte[] {
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9
		};

		muLawCompressTableChunks[166] = new byte[] {
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9
		};

		muLawCompressTableChunks[167] = new byte[] {
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9
		};

		muLawCompressTableChunks[168] = new byte[] {
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,
			(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0x9,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa
		};

		muLawCompressTableChunks[169] = new byte[] {
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa
		};

		muLawCompressTableChunks[170] = new byte[] {
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa
		};

		muLawCompressTableChunks[171] = new byte[] {
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa
		};

		muLawCompressTableChunks[172] = new byte[] {
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,
			(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xa,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb
		};

		muLawCompressTableChunks[173] = new byte[] {
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb
		};

		muLawCompressTableChunks[174] = new byte[] {
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb
		};

		muLawCompressTableChunks[175] = new byte[] {
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb
		};

		muLawCompressTableChunks[176] = new byte[] {
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,
			(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xb,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc
		};

		muLawCompressTableChunks[177] = new byte[] {
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc
		};

		muLawCompressTableChunks[178] = new byte[] {
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc
		};

		muLawCompressTableChunks[179] = new byte[] {
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc
		};

		muLawCompressTableChunks[180] = new byte[] {
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,
			(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xc,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd
		};

		muLawCompressTableChunks[181] = new byte[] {
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd
		};

		muLawCompressTableChunks[182] = new byte[] {
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd
		};

		muLawCompressTableChunks[183] = new byte[] {
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd
		};

		muLawCompressTableChunks[184] = new byte[] {
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,
			(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xd,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe
		};

		muLawCompressTableChunks[185] = new byte[] {
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe
		};

		muLawCompressTableChunks[186] = new byte[] {
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe
		};

		muLawCompressTableChunks[187] = new byte[] {
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe
		};

		muLawCompressTableChunks[188] = new byte[] {
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,
			(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xe,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf
		};

		muLawCompressTableChunks[189] = new byte[] {
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf
		};

		muLawCompressTableChunks[190] = new byte[] {
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf
		};

		muLawCompressTableChunks[191] = new byte[] {
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf
		};
	}
	
	private static void init192_223()
	{
		muLawCompressTableChunks[192] = new byte[] {
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,
			(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0xf,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10
		};

		muLawCompressTableChunks[193] = new byte[] {
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10
		};

		muLawCompressTableChunks[194] = new byte[] {
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,
			(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x10,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11
		};

		muLawCompressTableChunks[195] = new byte[] {
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11
		};

		muLawCompressTableChunks[196] = new byte[] {
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,
			(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x11,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12
		};

		muLawCompressTableChunks[197] = new byte[] {
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12
		};

		muLawCompressTableChunks[198] = new byte[] {
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,
			(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x12,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13
		};

		muLawCompressTableChunks[199] = new byte[] {
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13
		};

		muLawCompressTableChunks[200] = new byte[] {
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,
			(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x13,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14
		};

		muLawCompressTableChunks[201] = new byte[] {
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14
		};

		muLawCompressTableChunks[202] = new byte[] {
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,
			(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x14,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15
		};

		muLawCompressTableChunks[203] = new byte[] {
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15
		};

		muLawCompressTableChunks[204] = new byte[] {
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,
			(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x15,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16
		};

		muLawCompressTableChunks[205] = new byte[] {
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16
		};

		muLawCompressTableChunks[206] = new byte[] {
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,
			(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x16,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17
		};

		muLawCompressTableChunks[207] = new byte[] {
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17
		};

		muLawCompressTableChunks[208] = new byte[] {
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,
			(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x17,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18
		};

		muLawCompressTableChunks[209] = new byte[] {
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18
		};

		muLawCompressTableChunks[210] = new byte[] {
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,
			(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x18,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19
		};

		muLawCompressTableChunks[211] = new byte[] {
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19
		};

		muLawCompressTableChunks[212] = new byte[] {
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,
			(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x19,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a
		};

		muLawCompressTableChunks[213] = new byte[] {
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a
		};

		muLawCompressTableChunks[214] = new byte[] {
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,
			(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1a,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b
		};

		muLawCompressTableChunks[215] = new byte[] {
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b
		};

		muLawCompressTableChunks[216] = new byte[] {
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,
			(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1b,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c
		};

		muLawCompressTableChunks[217] = new byte[] {
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c
		};

		muLawCompressTableChunks[218] = new byte[] {
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,
			(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1c,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d
		};

		muLawCompressTableChunks[219] = new byte[] {
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d
		};

		muLawCompressTableChunks[220] = new byte[] {
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,
			(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1d,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e
		};

		muLawCompressTableChunks[221] = new byte[] {
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e
		};

		muLawCompressTableChunks[222] = new byte[] {
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,
			(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1e,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f
		};

		muLawCompressTableChunks[223] = new byte[] {
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f
		};
	}
	
	private static void init224_255()
	{
		muLawCompressTableChunks[224] = new byte[] {
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,
			(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x1f,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20
		};

		muLawCompressTableChunks[225] = new byte[] {
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,
			(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x20,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21
		};

		muLawCompressTableChunks[226] = new byte[] {
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,
			(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x21,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22
		};

		muLawCompressTableChunks[227] = new byte[] {
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,
			(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x22,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23
		};

		muLawCompressTableChunks[228] = new byte[] {
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,
			(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x23,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24
		};

		muLawCompressTableChunks[229] = new byte[] {
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,
			(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x24,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25
		};

		muLawCompressTableChunks[230] = new byte[] {
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,
			(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x25,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26
		};

		muLawCompressTableChunks[231] = new byte[] {
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,
			(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x26,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27
		};

		muLawCompressTableChunks[232] = new byte[] {
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,
			(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x27,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28
		};

		muLawCompressTableChunks[233] = new byte[] {
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,
			(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x28,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29
		};

		muLawCompressTableChunks[234] = new byte[] {
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,
			(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x29,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a
		};

		muLawCompressTableChunks[235] = new byte[] {
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,
			(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2a,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b
		};

		muLawCompressTableChunks[236] = new byte[] {
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,
			(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2b,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c
		};

		muLawCompressTableChunks[237] = new byte[] {
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,
			(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2c,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d
		};

		muLawCompressTableChunks[238] = new byte[] {
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,
			(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2d,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e
		};

		muLawCompressTableChunks[239] = new byte[] {
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,
			(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2e,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f
		};

		muLawCompressTableChunks[240] = new byte[] {
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,
			(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x2f,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30
		};

		muLawCompressTableChunks[241] = new byte[] {
			(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x30,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,
			(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x31,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32
		};

		muLawCompressTableChunks[242] = new byte[] {
			(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x32,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,
			(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x33,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34
		};

		muLawCompressTableChunks[243] = new byte[] {
			(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x34,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,
			(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x35,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36
		};

		muLawCompressTableChunks[244] = new byte[] {
			(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x36,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,
			(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x37,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38
		};

		muLawCompressTableChunks[245] = new byte[] {
			(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x38,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,
			(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x39,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a
		};

		muLawCompressTableChunks[246] = new byte[] {
			(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3a,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,
			(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3b,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c
		};

		muLawCompressTableChunks[247] = new byte[] {
			(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3c,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,
			(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3d,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e
		};

		muLawCompressTableChunks[248] = new byte[] {
			(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3e,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,
			(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x3f,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,
			(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x40,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41
		};

		muLawCompressTableChunks[249] = new byte[] {
			(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x41,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,
			(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x42,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,
			(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x43,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,
			(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x44,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45
		};

		muLawCompressTableChunks[250] = new byte[] {
			(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x45,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,
			(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x46,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,
			(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x47,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,
			(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x48,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49
		};

		muLawCompressTableChunks[251] = new byte[] {
			(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x49,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,
			(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4a,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,
			(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4b,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,
			(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4c,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d
		};

		muLawCompressTableChunks[252] = new byte[] {
			(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4d,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,
			(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4e,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,
			(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x4f,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,
			(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x50,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,
			(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x51,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,
			(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x52,(byte)0x53,(byte)0x53,(byte)0x53,
			(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,
			(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,
			(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53
		};

		muLawCompressTableChunks[253] = new byte[] {
			(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x53,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,
			(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x54,(byte)0x55,(byte)0x55,(byte)0x55,
			(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,
			(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,
			(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,
			(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x55,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,
			(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x56,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,
			(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x57,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,
			(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x58,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,
			(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x59,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,
			(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5a,(byte)0x5b,(byte)0x5b,(byte)0x5b,
			(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,
			(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,
			(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b
		};

		muLawCompressTableChunks[254] = new byte[] {
			(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5b,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,
			(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5c,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,
			(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5d,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,
			(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5e,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,
			(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x5f,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,
			(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x60,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,
			(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x61,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,
			(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x62,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,
			(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x63,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,
			(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x64,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,
			(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x65,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,
			(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x66,(byte)0x67,(byte)0x67,(byte)0x67,
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67
		};

		muLawCompressTableChunks[255] = new byte[] {
			(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x67,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,
			(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x68,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,
			(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x69,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,
			(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6a,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,
			(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6b,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,
			(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6c,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,
			(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6d,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,
			(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6e,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,
			(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x6f,(byte)0x70,(byte)0x70,(byte)0x70,
			(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x70,(byte)0x71,(byte)0x71,(byte)0x71,
			(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x71,(byte)0x72,(byte)0x72,(byte)0x72,
			(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x72,(byte)0x73,(byte)0x73,(byte)0x73,
			(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x73,(byte)0x74,(byte)0x74,(byte)0x74,
			(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x74,(byte)0x75,(byte)0x75,(byte)0x75,
			(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x75,(byte)0x76,(byte)0x76,(byte)0x76,
			(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x76,(byte)0x77,(byte)0x77,(byte)0x77,
			(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x77,(byte)0x78,(byte)0x78,(byte)0x78,
			(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x78,(byte)0x79,(byte)0x79,(byte)0x79,
			(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x79,(byte)0x7a,(byte)0x7a,(byte)0x7a,
			(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7a,(byte)0x7b,(byte)0x7b,(byte)0x7b,
			(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7b,(byte)0x7c,(byte)0x7c,(byte)0x7c,
			(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7c,(byte)0x7d,(byte)0x7d,(byte)0x7d,
			(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7d,(byte)0x7e,(byte)0x7e,(byte)0x7e,
			(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7e,(byte)0x7f,(byte)0x7f,(byte)0x7f
		};
	}
}